package net.wanji.opt.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.utils.tool.LocalDateTimeUtil;
import net.wanji.common.utils.tool.StringUtils;
import net.wanji.databus.bo.CrossIdBO;
import net.wanji.databus.dao.mapper.BaseCrossInfoMapper;
import net.wanji.databus.dao.mapper.CrossDataRealtimeMapper;
import net.wanji.databus.dao.mapper.CrossDirDataHistMapper;
import net.wanji.databus.dao.mapper.CrossDirDataRealtimeMapper;
import net.wanji.databus.po.*;
import net.wanji.opt.common.enums.CrossOptResultStrategyEnum;
import net.wanji.opt.common.enums.EventInfoTypeEnum;
import net.wanji.opt.common.enums.OptStatusEnum;
import net.wanji.opt.common.enums.StrategyControlEnum;
import net.wanji.opt.dao.mapper.HoloEventMapper;
import net.wanji.opt.dao.mapper.StrategyCrossResultMapper;
import net.wanji.opt.po.trend.HoloEventInfoPO;
import net.wanji.opt.service.CrossIndexService;
import net.wanji.opt.synthesis.pojo.StrategyCrossResultEntity;
import net.wanji.opt.vo.AIOptResultVO;
import net.wanji.opt.vo.CrossOptResult;
import net.wanji.opt.vo.CrossStatusCountVO;
import org.apache.curator.drivers.EventTrace;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.ZoneId;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author duanruiming
 * @date 2024/11/25 19:12
 */
@Service
@Slf4j
public class CrossIndexServiceImpl implements CrossIndexService {

    @Resource
    private CrossDirDataRealtimeMapper crossDirDataRealtimeMapper;
    @Resource
    private CrossDataRealtimeMapper crossDataRealtimeMapper;
    @Resource
    private BaseCrossInfoMapper baseCrossInfoMapper;
    @Resource
    private StrategyCrossResultMapper strategyCrossResultMapper;
    @Resource
    private HoloEventMapper holoEventMapper;
    @Resource
    private CrossDirDataHistMapper crossDirDataHistMapper;

    @Override
    public Map<Integer, CrossDirDataRealtimePO> crossDirIndex(CrossIdBO crossIdBO) {
        List<CrossDirDataRealtimePO> crossDirDataRealtimePOS = crossDirDataRealtimeMapper.selectBycrossId(crossIdBO.getCrossId());
        if (!CollectionUtils.isEmpty(crossDirDataRealtimePOS)) {
            Map<Integer, CrossDirDataRealtimePO> dirDataMap = new HashMap<>(crossDirDataRealtimePOS.size());
            for (CrossDirDataRealtimePO crossDirDataRealtimePO : crossDirDataRealtimePOS) {
                dirDataMap.put(crossDirDataRealtimePO.getDirType(), crossDirDataRealtimePO);
            }
            return dirDataMap;
        }
        return null;
    }

    @Override
    public CrossDataRealtimePO crossIndex(CrossIdBO crossIdBO) {
        CrossDataRealtimePO crossDataRealtimePO = crossDataRealtimeMapper.selectByCrossId(crossIdBO.getCrossId());
        return crossDataRealtimePO;
    }


    @Override
    public List<TBaseCrossInfo> crossInfoList() {
        LambdaQueryWrapper<TBaseCrossInfo> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(TBaseCrossInfo::getIsSignal, 1);
        List<TBaseCrossInfo> baseCrossInfoPOS = baseCrossInfoMapper.selectList(queryWrapper);
        return baseCrossInfoPOS;
    }

    @Override
    public List<CrossOptResult> crossOptResultList(CrossIdBO crossIdBO) throws Exception {
        LambdaQueryWrapper<StrategyCrossResultEntity> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(StrategyCrossResultEntity::getCrossId, crossIdBO.getCrossId());
        LocalDate currentDate = LocalDate.now();
        LocalTime startTime = LocalTime.MIDNIGHT;
        LocalDateTime startOfDay = LocalDateTime.of(currentDate, startTime);
        queryWrapper.eq(StrategyCrossResultEntity::getCrossId, crossIdBO.getCrossId());
        queryWrapper.ge(StrategyCrossResultEntity::getIssueTime, startOfDay);
        List<StrategyCrossResultEntity> list = strategyCrossResultMapper.selectList(queryWrapper);
        List<CrossOptResult> crossOptResults = new ArrayList<>();
        if (!CollectionUtils.isEmpty(list)) {
            for (StrategyCrossResultEntity entity : list) {
                CrossOptResult crossOptResult = new CrossOptResult();
                Date date = entity.getIssueTime();
                crossOptResult.setTimeStamp(date);
                Integer countDown = entity.getCountDown();
                if (entity.getCurrentAlgo() == 2) {
                    countDown = 5;
                }
                crossOptResult.setCountDown(countDown);
                Integer currentAlgo = entity.getCurrentAlgo();
                crossOptResult.setStrategy(currentAlgo);
                crossOptResult.setStrategyName(CrossOptResultStrategyEnum.getDesc(currentAlgo));
                crossOptResults.add(crossOptResult);
            }
        }
        return crossOptResults;
    }

    @Override
    public List<AIOptResultVO> crossAIList() {
        List<AIOptResultVO> results = new ArrayList<>();
        try {
            LocalDate currentDate = LocalDate.now();
            LocalTime startTime = LocalTime.MIDNIGHT;
            LocalDateTime startOfDay = LocalDateTime.of(currentDate, startTime);
            Date date = Date.from(startOfDay.atZone(ZoneId.systemDefault()).toInstant());
            List<StrategyCrossResultEntity> list = strategyCrossResultMapper.selectAICrossList(date);
            if (!CollectionUtils.isEmpty(list)) {
                for (StrategyCrossResultEntity resultEntity : list) {
                    AIOptResultVO aiOptResultVO = new AIOptResultVO();
                    aiOptResultVO.setId(resultEntity.getCrossId());
                    aiOptResultVO.setName(resultEntity.getCrossName());
                    Integer currentAlgo = resultEntity.getCurrentAlgo();
                    aiOptResultVO.setStrategy(currentAlgo);
                    // 失衡 均衡调控，其他效率提升
                    aiOptResultVO.setOptMethod(Objects.equals(1, currentAlgo) ? StrategyControlEnum.ONE.getMethod() : StrategyControlEnum.TWO.getMethod());
                    // 失衡 优化中，15分钟内，优化中，其他，优化完
                    aiOptResultVO.setOptStatus(OptStatusEnum.TWO.getCode());
                    if (Objects.equals(1, currentAlgo)) {
                        Date issueTimeDate = resultEntity.getIssueTime();
                        //Date issueTimeDate = DateUtils.parseDate(issueTime);
                        long optTime = issueTimeDate.getTime();
                        long currentTimeMillis = System.currentTimeMillis();
                        long offset = currentTimeMillis - optTime;
                        if (offset <= 15 * 60 * 1000) {
                            aiOptResultVO.setOptStatus(OptStatusEnum.ONE.getCode());
                        }
                    }
                    aiOptResultVO.setOptStatusName(Objects.equals(1, currentAlgo) ? OptStatusEnum.ONE.getDesc() : OptStatusEnum.TWO.getDesc());
                    if (Objects.isNull(currentAlgo)) {
                        aiOptResultVO.setOptStatus(-1);
                        aiOptResultVO.setOptStatusName(OptStatusEnum.ZERO.getDesc());
                    }
                    results.add(aiOptResultVO);
                }
            }
        } catch (Exception e) {
            log.error("AI路口监测查询异常：", e);
            throw new RuntimeException(e);
        }
        Collections.sort(results, Comparator.comparingInt(AIOptResultVO::getOptStatus).reversed());
        return results;
    }

    @Override
    public List<CrossStatusCountVO> crossStatusCount(String crossId) throws Exception {
        List<CrossStatusCountVO> results = new ArrayList<>();

        try {
            LocalDate currentDate = LocalDate.now();
            LocalTime startTime = LocalTime.MIDNIGHT;
            LocalDateTime startOfDay = LocalDateTime.of(currentDate, startTime);
            LambdaQueryWrapper<HoloEventInfoPO> queryWrapper = new LambdaQueryWrapper<>();
            queryWrapper.eq(HoloEventInfoPO::getCrossId, crossId);
            queryWrapper.ge(HoloEventInfoPO::getStartTime, startOfDay);
            List<HoloEventInfoPO> holoEventInfoPOS = holoEventMapper.selectList(queryWrapper);
            if (!CollectionUtils.isEmpty(holoEventInfoPOS)) {
                Map<String, List<HoloEventInfoPO>> eventTypeMap = holoEventInfoPOS.stream().collect(Collectors.groupingBy(HoloEventInfoPO::getType));
                for (Map.Entry<String, List<HoloEventInfoPO>> entry : eventTypeMap.entrySet()) {
                    String eventType = entry.getKey();
                    List<HoloEventInfoPO> value = entry.getValue();
                    // 过滤干线
                    if (StringUtils.equalsIgnoreCase(eventType, EventInfoTypeEnum.GREEN_SLOW.getEventType())
                            || StringUtils.equalsIgnoreCase(eventType, EventInfoTypeEnum.GREEN_CONGEST.getEventType())) {
                        continue;
                    }
                    // 当前事件返回实体
                    CrossStatusCountVO crossStatusCountVO = new CrossStatusCountVO();
                    crossStatusCountVO.setType(EventInfoTypeEnum.getOptType(eventType));
                    crossStatusCountVO.setCount(value.size());
                    List<CrossStatusCountVO.DirTurn> dirTurns = new ArrayList<>();
                    if (!CollectionUtils.isEmpty(value)) {
                        for (HoloEventInfoPO holoEventInfoPO : value) {
                            CrossStatusCountVO.DirTurn dirTurn = new CrossStatusCountVO.DirTurn();
                            dirTurn.setDir(holoEventInfoPO.getDir());
                            dirTurn.setTurn(holoEventInfoPO.getTurn());
                            dirTurns.add(dirTurn);
                        }
                    }
                    crossStatusCountVO.setList(dirTurns);
                    results.add(crossStatusCountVO);
                }
            }
        } catch (Exception e) {
            log.error("优化监测-问题诊断查询异常：", e);
            throw new RuntimeException(e);
        }
        return results;
    }

    @Override
    public CrossDirStatusDataPO selectByCrossIdAndHour(String crossId, int hour) {
        List<CrossDirDataHistPO> crossDirDataHistPOS = crossDirDataHistMapper.selectByCrossIdAndHour(crossId, hour);
        CrossDirStatusDataPO crossDirStatusDataPO = new CrossDirStatusDataPO();

        if (ObjectUtil.isNotEmpty(crossDirDataHistPOS)){
            //LocalDateTimeUtil.formatTimeStamp(Long.valueOf(x.getBatchTime())*1000, "HH:mm")
            //根据dirType升序，分组
            LinkedHashMap<Integer, List<CrossDirDataHistPO>> stringListMap = crossDirDataHistPOS.stream().sorted(Comparator.comparing(CrossDirDataHistPO::getDirType)).collect(Collectors.groupingBy(x -> x.getDirType(), LinkedHashMap::new, Collectors.toList()));
            Set<Integer> stringSet = stringListMap.keySet();
            Map<Integer, List<Integer>> dirStatusListMap = new HashMap<>();
            for (Integer dir : stringSet) {
                List<Integer> list = stringListMap.get(dir).stream().sorted(Comparator.comparing(CrossDirDataHistPO::getBatchTime)).map(x -> x.getStatus()).collect(Collectors.toList());
                dirStatusListMap.put(dir,list);
            }
            //获取第一个方向
            Integer firstDir = stringSet.iterator().next();
            List<String> timeList = stringListMap.get(firstDir).stream().sorted(Comparator.comparing(CrossDirDataHistPO::getBatchTime)).map(x -> LocalDateTimeUtil.formatTimeStamp(Long.valueOf(x.getBatchTime()) * 1000, "HH:mm")).collect(Collectors.toList());
            crossDirStatusDataPO.setTimeList(timeList);
            crossDirStatusDataPO.setDirStatusListMap(dirStatusListMap);
        }
        return crossDirStatusDataPO;
    }


    @Override
    public HoloEventInfoPO crossEventDirTurn(String crossId) throws Exception {
        try {
            LambdaQueryWrapper<HoloEventInfoPO> queryWrapper = new LambdaQueryWrapper<>();
            queryWrapper.eq(HoloEventInfoPO::getCrossId, crossId);
            queryWrapper.orderByDesc(HoloEventInfoPO::getStartTime);
            queryWrapper.last("limit 1");
            HoloEventInfoPO holoEventInfoPO = holoEventMapper.selectOne(queryWrapper);
            if (Objects.nonNull(holoEventInfoPO)) {
                return holoEventInfoPO;
            }
            return null;
        } catch (Exception e) {
            log.error("交通体检-路口监测-事件方向转向 查询失败：", e);
            throw new Exception(e);
        }
    }
}
