package net.wanji.opt.service.impl;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.github.pagehelper.PageInfo;
import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.enums.LightsAddressEnum;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.common.utils.tool.CrossUtil;
import net.wanji.common.utils.tool.JacksonUtils;
import net.wanji.databus.bo.CrossIdAndSchemeIdBO;
import net.wanji.databus.bo.CrossIdBO;
import net.wanji.databus.dao.entity.*;
import net.wanji.databus.dao.mapper.*;
import net.wanji.databus.dto.CrossSchemeRingsDTO;
import net.wanji.databus.dto.QueryByCrossIdAndTimeDTO;
import net.wanji.databus.po.*;
import net.wanji.databus.vo.SchemeOptSendVO;
import net.wanji.databus.vo.SchemeSendVO;
import net.wanji.feign.service.UtcFeignClients;
import net.wanji.opt.dao.mapper.CrossSchemeOptLogMapper;
import net.wanji.opt.dao.mapper.strategy.SceneMapper;
import net.wanji.opt.dto.*;
import net.wanji.opt.dto.strategy.AddOrUpdateSceneDTO;
import net.wanji.opt.dto.strategy.QuerySceneDTO;
import net.wanji.opt.po.base.CrossSchemeOptLogPO;
import net.wanji.opt.po.strategy.ScenePO;
import net.wanji.opt.service.DiagnoService;
import net.wanji.opt.service.signalcontrol.FeignProxyService;
import net.wanji.opt.service.strategy.SceneService;
import net.wanji.opt.vo.CrossIdAndLocationVO;
import net.wanji.opt.vo.OptEffectVO;
import net.wanji.opt.vo.SchemeOptVO;
import org.apache.commons.lang3.ObjectUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author Kent HAN
 * @date 2023/3/2 10:09
 */
@Service
@Slf4j
public class DiagnoServiceImpl implements DiagnoService {

    private final BaseCrossInfoMapper baseCrossInfoMapper;
    private final CrossDataRealtimeMapper crossDataRealtimeMapper;
    private final SceneService sceneService;
    private final SceneMapper sceneMapper;
    private final BaseCrossDirInfoMapper baseCrossDirInfoMapper;
    private final RidInfoMapper ridInfoMapper;
    private final CrossSchemeOptLogMapper crossSchemeOptLogMapper;
    private final BaseCrossSchemeMapper baseCrossSchemeMapper;
    private final BaseCrossPhaseMapper baseCrossPhaseMapper;
    private final FeignProxyService feignProxyService;
    private final CrossDataHistMapper crossDataHistMapper;
    private final CrossDirDataHistMapper crossDirDataHistMapper;
    private final LaneInfoMapper laneInfoMapper;
    private final CrossDirDataRealtimeMapper crossDirDataRealtimeMapper;
    private final BaseCrossLaneLightsMapper baseCrossLaneLightsMapper;
    private final BaseCrossLightsMapper baseCrossLightsMapper;
    @Resource
    private UtcFeignClients utcFeignClients;
    private Gson gson = new GsonBuilder().setDateFormat("yyyy-MM-dd HH:mm:ss").create();

    // 机动车灯数据库类型与前端类型对应关系
    private static final Map<Integer, Integer> MOTOR_VEH_MAP = new HashMap<>(5);

    static {
        MOTOR_VEH_MAP.put(1, 1);
        MOTOR_VEH_MAP.put(2, 3);
        MOTOR_VEH_MAP.put(3, 4);
        MOTOR_VEH_MAP.put(4, 5);
        MOTOR_VEH_MAP.put(5, 6);
    }

    // 非机动车灯数据库类型与前端类型对应关系
    private static final Map<Integer, Integer> NON_MOTOR_VEH_MAP = new HashMap<>(5);

    static {
        MOTOR_VEH_MAP.put(10, 2);
        MOTOR_VEH_MAP.put(11, 3);
        MOTOR_VEH_MAP.put(12, 4);
        MOTOR_VEH_MAP.put(13, 5);
        MOTOR_VEH_MAP.put(14, 6);
    }

    public DiagnoServiceImpl(BaseCrossInfoMapper baseCrossInfoMapper, CrossDataRealtimeMapper crossDataRealtimeMapper,
                             SceneService sceneService, SceneMapper sceneMapper, BaseCrossDirInfoMapper baseCrossDirInfoMapper,
                             RidInfoMapper ridInfoMapper, CrossSchemeOptLogMapper crossSchemeOptLogMapper,
                             BaseCrossSchemeMapper baseCrossSchemeMapper, BaseCrossPhaseMapper baseCrossPhaseMapper,
                             FeignProxyService feignProxyService, CrossDataHistMapper crossDataHistMapper,
                             CrossDirDataHistMapper crossDirDataHistMapper, LaneInfoMapper laneInfoMapper,
                             CrossDirDataRealtimeMapper crossDirDataRealtimeMapper,
                             BaseCrossLaneLightsMapper baseCrossLaneLightsMapper, BaseCrossLightsMapper baseCrossLightsMapper) {
        this.baseCrossInfoMapper = baseCrossInfoMapper;
        this.crossDataRealtimeMapper = crossDataRealtimeMapper;
        this.sceneService = sceneService;
        this.sceneMapper = sceneMapper;
        this.baseCrossDirInfoMapper = baseCrossDirInfoMapper;
        this.ridInfoMapper = ridInfoMapper;
        this.crossSchemeOptLogMapper = crossSchemeOptLogMapper;
        this.baseCrossSchemeMapper = baseCrossSchemeMapper;
        this.baseCrossPhaseMapper = baseCrossPhaseMapper;
        this.feignProxyService = feignProxyService;
        this.crossDataHistMapper = crossDataHistMapper;
        this.crossDirDataHistMapper = crossDirDataHistMapper;
        this.laneInfoMapper = laneInfoMapper;
        this.crossDirDataRealtimeMapper = crossDirDataRealtimeMapper;
        this.baseCrossLaneLightsMapper = baseCrossLaneLightsMapper;
        this.baseCrossLightsMapper = baseCrossLightsMapper;
    }

    @Override
    public AddOrUpdateSceneDTO queryCrossScene(CrossIdBO crossIdBO) {
        // 查询路口场景代码
        String crossId = crossIdBO.getCrossId();
        CrossDataRealtimePO crossDataRealtimePO = crossDataRealtimeMapper.selectByCrossId(crossId);
        Integer status = crossDataRealtimePO.getStatus();
        // 获取场景名称
        ScenePO scenePO = sceneMapper.selectBySceneNum(status);
        String sceneName = scenePO.getSceneName();
        // 构造请求体
        QuerySceneDTO querySceneDTO = new QuerySceneDTO();
        querySceneDTO.setSceneName(sceneName);
        querySceneDTO.setSceneTarget(1);
        querySceneDTO.setPageNum(1);
        querySceneDTO.setPageSize(999);
        PageInfo<AddOrUpdateSceneDTO> pageInfo = sceneService.queryScene(querySceneDTO);
        List<AddOrUpdateSceneDTO> list = pageInfo.getList();

        return list.get(0);
    }

    @Override
    public List<CrossIdAndLocationVO> queryNeighborCross(CrossIdBO crossIdBO) {
        List<CrossIdAndLocationVO> crossIdAndLocationVOList = new ArrayList<>();
        String crossId = crossIdBO.getCrossId();
        List<Integer> inDirs = baseCrossDirInfoMapper.selectInDirsByCrossId(crossId);
        for (Integer inDir : inDirs) {
            RidInfoEntity ridInfo = ridInfoMapper.selectByEndInDir(crossId, inDir);
            if (ridInfo != null) {
                CrossIdAndLocationVO crossIdAndLocationVO = new CrossIdAndLocationVO();
                // 关联路口坐标
                String startCrossId = ridInfo.getStartCrossId();
                crossIdAndLocationVO.setId(startCrossId);
                TBaseCrossInfo crossInfoPO = baseCrossInfoMapper.selectByPrimaryKey(startCrossId);
                if (crossInfoPO != null) {
                    crossIdAndLocationVO.setName(crossInfoPO.getName());
                    String startLocationStr = crossInfoPO.getLocation();
                    double[] startLonLat = CrossUtil.getLonLat(startLocationStr);
                    List<Double> startLocation = new ArrayList<>();
                    startLocation.add(startLonLat[0]);
                    startLocation.add(startLonLat[1]);
                    crossIdAndLocationVO.setLocation(startLocation);
                }
                // 关联路口wkt
                String wkt = ridInfo.getWkt();
                crossIdAndLocationVO.setWkt(wkt);
                CrossDataRealtimePO crossDataRealtimePO = crossDataRealtimeMapper.selectByCrossId(startCrossId);
                if (crossDataRealtimePO == null) {
                    throw new RuntimeException("没有该路口实时数据");
                }
                crossIdAndLocationVO.setRealtimeStatus(crossDataRealtimePO.getStatus());
                crossIdAndLocationVOList.add(crossIdAndLocationVO);
            }
        }
        return crossIdAndLocationVOList;
    }

    @Override
    public SchemeOptVO querySchemeOpt(CrossIdAndSchemeIdBO bo) {
        String crossId = bo.getCrossId();
        Integer schemeId = bo.getSchemeId();
        SchemeOptVO schemeOptVO = new SchemeOptVO();
        schemeOptVO.setCrossId(crossId);
        schemeOptVO.setSchemeId(schemeId);
        // 选择数据批次时间最大的记录
        List<CrossSchemeOptLogPO> maxList = crossSchemeOptLogMapper.selectMaxByCrossIdAndSchemeId(crossId, schemeId);
        if (!CollectionUtils.isEmpty(maxList)) {
            // 根据相位去重
            TreeSet<CrossSchemeOptLogPO> collect = maxList.stream()
                    .collect(Collectors.toCollection(
                            () -> new TreeSet<>(Comparator.comparing(CrossSchemeOptLogPO::getPhaseNo))));
            CrossSchemeOptLogPO crossSchemeOptLogPO = maxList.get(0);
            String optType = crossSchemeOptLogPO.getOptType();
            schemeOptVO.setOptType(Integer.parseInt(optType));
            BaseCrossSchemePO baseCrossSchemePO = baseCrossSchemeMapper.selectByCrossIdAndSchemeId(crossId, schemeId);
            if (baseCrossSchemePO != null) {
                schemeOptVO.setCycle(baseCrossSchemePO.getCycle());
                schemeOptVO.setOffset(crossSchemeOptLogPO.getOffset());
            }
            // 构造调整时间列表
            List<SchemeOptVO.PhaseOptTime> phaseOptTimeList = buildPhaseOptTimeList(collect, schemeId, crossId);
            schemeOptVO.setPhaseOptTimeList(phaseOptTimeList);
        } else {
            List<SchemeOptVO.PhaseOptTime> phaseOptTimeList = new ArrayList<>();
            initPhaseOptTimeList(schemeId, phaseOptTimeList, crossId);
            schemeOptVO.setPhaseOptTimeList(phaseOptTimeList);
        }
        return schemeOptVO;
    }

    @Override
    public List<SchemeOptVO.PhaseOptTime> queryOptTime(CrossIdAndSchemeIdDTO crossIdAndSchemeIdDTO) {
        String crossId = crossIdAndSchemeIdDTO.getCrossId();
        Integer schemeId = crossIdAndSchemeIdDTO.getSchemeId();
        List<SchemeOptVO.PhaseOptTime> phaseOptTimeList = new ArrayList<>();
        // 选择数据批次时间最大的记录
        List<CrossSchemeOptLogPO> maxList = crossSchemeOptLogMapper.selectMaxByCrossId(crossId);
        for (CrossSchemeOptLogPO crossSchemeOptLogPO : maxList) {
            SchemeOptVO.PhaseOptTime phaseOptTime = new SchemeOptVO.PhaseOptTime();
            String phaseNo = crossSchemeOptLogPO.getPhaseNo();
            phaseOptTime.setPhaseNo(phaseNo);
            Integer oriGreenTime = crossSchemeOptLogPO.getOriGreenTime();
            phaseOptTime.setOriGreenTime(oriGreenTime);
            Integer optGreenTime = getNewGreenTime(crossId, schemeId, phaseNo);
            phaseOptTime.setOptGreenTime(optGreenTime);
            phaseOptTime.setOptTime(optGreenTime - oriGreenTime);
            phaseOptTimeList.add(phaseOptTime);
        }
        return phaseOptTimeList;
    }

    @Override
    public List<SchemeOptVO.PhaseOptTime> schemeComparison(CrossIdAndSchemeIdDTO crossIdAndSchemeIdDTO) {
        String crossId = crossIdAndSchemeIdDTO.getCrossId();
        List<SchemeOptVO.PhaseOptTime> phaseOptTimeList = new ArrayList<>();
        QueryByCrossIdAndTimeDTO queryByCrossIdAndTimeDTO = new QueryByCrossIdAndTimeDTO();
        queryByCrossIdAndTimeDTO.setCrossId(crossId);
        queryByCrossIdAndTimeDTO.setDatetime(new Date());
        //查询路口当前方案
        JsonViewObject jsonViewObjectScheme = utcFeignClients.selectSchemeByParams(queryByCrossIdAndTimeDTO);
        Integer jsonViewObjectSchemeCode = jsonViewObjectScheme.getCode();
        if (jsonViewObjectSchemeCode != 200){
            log.info("未查询到当前路口号,crossId:{}",crossId);
            return phaseOptTimeList;
        }
        ObjectMapper mapper1 = JacksonUtils.getInstance();
        BaseCrossSchemePO baseCrossSchemePO = mapper1.convertValue(jsonViewObjectScheme.getContent(), new TypeReference<BaseCrossSchemePO>() {
        });

        if (ObjectUtils.isEmpty(baseCrossSchemePO)){
            return phaseOptTimeList;
        }
        Integer schemeNo = Integer.valueOf(baseCrossSchemePO.getSchemeNo());
        if (ObjectUtil.isEmpty(schemeNo) || schemeNo == 85){
            log.info("当前路口为黃闪,crossId:{}",crossId);
            return phaseOptTimeList;
        }
        List<CrossPhasePO> phasePOList = baseCrossPhaseMapper.selectByCrossIdAndSchemeNo(crossId, schemeNo);
        if (ObjectUtils.isEmpty(phasePOList) || ObjectUtils.isEmpty(schemeNo)) {
            return phaseOptTimeList;
        }
        CrossSchemeRingsDTO crossSchemeRingsDTO = new CrossSchemeRingsDTO();
        crossSchemeRingsDTO.setCrossId(crossId);
        crossSchemeRingsDTO.setPattern(String.valueOf(schemeNo));

        JsonViewObject jsonViewObject = utcFeignClients.schemeOptLog(crossSchemeRingsDTO);
        Integer code = jsonViewObject.getCode();
        ObjectMapper mapper = JacksonUtils.getInstance();
        List<CrossSchemeStageOptLogPO> crossSchedulesPOList = mapper.convertValue(jsonViewObject.getContent(), new TypeReference<List<CrossSchemeStageOptLogPO>>() {
        });
        //
        if (code != 200 || ObjectUtils.isEmpty(crossSchedulesPOList)) {
            log.info("该路口方案暂未优化,{}", JSONObject.toJSONString(crossIdAndSchemeIdDTO));
            //将优化后的方案都设置为0
            for (CrossPhasePO crossPhasePO : phasePOList) {
                SchemeOptVO.PhaseOptTime phaseOptTime = new SchemeOptVO.PhaseOptTime();
                phaseOptTime.setPhaseNo(crossPhasePO.getPhaseNo());
                phaseOptTime.setOriGreenTime(crossPhasePO.getPhaseTime());
                phaseOptTime.setOptGreenTime(0);
                phaseOptTime.setOptTime(0);
                phaseOptTimeList.add(phaseOptTime);
            }
        } else {
            for (CrossPhasePO crossPhasePO : phasePOList) {
                SchemeOptVO.PhaseOptTime phaseOptTime = new SchemeOptVO.PhaseOptTime();
                phaseOptTime.setPhaseNo(String.valueOf(crossPhasePO.getSort()));
                Integer oriPhaseTime = crossPhasePO.getPhaseTime();
                phaseOptTime.setOriGreenTime(oriPhaseTime);
                List<CrossSchemeStageOptLogPO> list = crossSchedulesPOList.stream().filter(x -> x.getPhaseNo().equals(String.valueOf(crossPhasePO.getSort()))).collect(Collectors.toList());
                if (ObjectUtils.isEmpty(list)) {
                    phaseOptTime.setOptGreenTime(0);
                    phaseOptTime.setOptTime(0-oriPhaseTime);
                } else {
                    CrossSchemeStageOptLogPO optLogPO = list.get(0);
                    Integer optPhaseTime = optLogPO.getPhaseTime();
                    phaseOptTime.setOptGreenTime(optPhaseTime);
                    int optTime = optPhaseTime - oriPhaseTime;
                    phaseOptTime.setOptTime(optTime);
                }
                phaseOptTimeList.add(phaseOptTime);
            }
        }
        return phaseOptTimeList;
    }

    @Override
    public void sendManual(SendManualDTO sendManualDTO) throws Exception {
        // 构造场景、策略、方法
        String optDataExtend = buildOptDataExtend(sendManualDTO);
        SchemeOptSendVO schemeOptSendVO = new SchemeOptSendVO();
        schemeOptSendVO.setDataExtend(optDataExtend);
        Integer optTypeInt = sendManualDTO.getOptType();
        String optTypeStr = String.valueOf(optTypeInt);
        schemeOptSendVO.setOptType(optTypeStr);
        Integer crossStatus = sendManualDTO.getCrossStatus();
        String crossStatusStr = String.valueOf(crossStatus);
        schemeOptSendVO.setOptReason(crossStatusStr);

        String currentCrossId = sendManualDTO.getCurrentCrossId();
        String sendCrossId = sendManualDTO.getSendCrossId();
        if (!Objects.equals(currentCrossId, sendCrossId)) {
            schemeOptSendVO.setRelationFlag(String.valueOf(0)); // 是否被关联调整 1否 0是
            schemeOptSendVO.setRelationCrossId(currentCrossId);
        } else {
            schemeOptSendVO.setRelationFlag(String.valueOf(1));
        }

        Map<String, Integer> phaseOffsetTimeMap = new HashMap<>();
        List<SchemeOptVO.PhaseOptTime> phaseOptTimeList = sendManualDTO.getPhaseOptTimeList();
        for (SchemeOptVO.PhaseOptTime phaseOptTime : phaseOptTimeList) {
            phaseOffsetTimeMap.put(phaseOptTime.getPhaseNo(), phaseOptTime.getOptTime());
        }
        schemeOptSendVO.setPhaseOffsetTimeMap(phaseOffsetTimeMap);
        schemeOptSendVO.setCrossCode(sendCrossId);
        Integer schemeId = sendManualDTO.getSchemeId();
        List<SchemeSendVO.Pattern> patternList = buildPatternList(sendCrossId, schemeId, phaseOffsetTimeMap);
        schemeOptSendVO.setPatternList(patternList);

        feignProxyService.schemeOptSend(schemeOptSendVO);
    }

    private String buildOptDataExtend(SendManualDTO sendManualDTO) {
        String crossId = sendManualDTO.getSendCrossId();
        // 选择数据批次时间最大的记录
        List<CrossSchemeOptLogPO> maxList = crossSchemeOptLogMapper.selectMaxByCrossId(crossId);
        if (!CollectionUtils.isEmpty(maxList)) {
            CrossSchemeOptLogPO crossSchemeOptLogPO = maxList.get(0);
            String dataExtend = crossSchemeOptLogPO.getDataExtend();
            OptDataExtend optDataExtendOld = gson.fromJson(dataExtend, OptDataExtend.class);
            OptDataExtend optDataExtendNew = new OptDataExtend();
            optDataExtendNew.setSceneName(optDataExtendOld.getSceneName());
            optDataExtendNew.setStrategyName(optDataExtendOld.getStrategyName());
            // todo 改为枚举值
            optDataExtendNew.setIdeaName("手动干预");
            return gson.toJson(optDataExtendNew);
        }
        return null;
    }

    private List<SchemeSendVO.Pattern> buildPatternList(String sendCrossId, Integer schemeId,
                                                        Map<String, Integer> phaseOffsetTimeMap) {
        List<SchemeSendVO.Pattern> patternList = new ArrayList<>();
        BaseCrossSchemePO baseCrossSchemePO = baseCrossSchemeMapper.selectById(schemeId);
        SchemeSendVO.Pattern pattern = new SchemeSendVO.Pattern();
        pattern.setPatternNo(baseCrossSchemePO.getSchemeNo());
        pattern.setPatternName(baseCrossSchemePO.getName());
        pattern.setCycle(baseCrossSchemePO.getCycle().toString());
        pattern.setCoordPhase(baseCrossSchemePO.getCoordPhase().toString());
        pattern.setOffset(baseCrossSchemePO.getOffset().toString());
        // 构造环列表
        List<SchemeSendVO.Pattern.Ring> rings = buildRings(sendCrossId, schemeId, phaseOffsetTimeMap);
        pattern.setRings(rings);
        patternList.add(pattern);

        return patternList;
    }

    private List<SchemeSendVO.Pattern.Ring> buildRings(String sendCrossId, Integer schemeId,
                                                       Map<String, Integer> phaseOffsetTimeMap) {
        List<SchemeSendVO.Pattern.Ring> rings = new ArrayList<>();
        // 根据路口ID和方案ID查询相位集合，根据ringNo分组，特殊控制ringNo为0
        CrossPhasePO crossPhasePO = new CrossPhasePO();
        crossPhasePO.setCrossId(sendCrossId);
        crossPhasePO.setPlanId(schemeId);
        List<CrossPhasePO> crossPhasePOList = baseCrossPhaseMapper.listCrossPhasePO(crossPhasePO);
        Map<Integer, List<CrossPhasePO>> collectMap = crossPhasePOList.stream()
                .collect(Collectors.groupingBy(CrossPhasePO::getRingNo));
        for (Map.Entry<Integer, List<CrossPhasePO>> entry : collectMap.entrySet()) {
            SchemeSendVO.Pattern.Ring ring = new SchemeSendVO.Pattern.Ring();
            ring.setRingNo(entry.getKey().toString());
            // 构造相位列表
            List<CrossPhasePO> phases = entry.getValue();
            List<SchemeSendVO.Pattern.Ring.Phase> phaseList = buildPhaseList(phases, phaseOffsetTimeMap);
            ring.setPhaseList(phaseList);
            rings.add(ring);
        }
        return rings;
    }

    private List<SchemeSendVO.Pattern.Ring.Phase> buildPhaseList(List<CrossPhasePO> phases,
                                                                 Map<String, Integer> phaseOffsetTimeMap) {
        List<SchemeSendVO.Pattern.Ring.Phase> phaseList = new ArrayList<>();
        for (CrossPhasePO phasePO : phases) {
            SchemeSendVO.Pattern.Ring.Phase phase = new SchemeSendVO.Pattern.Ring.Phase();
            String phaseNo = phasePO.getPhaseNo();
            phase.setPhaseNo(phaseNo);
            phase.setPhaseName(phasePO.getName());
            phase.setSort(phasePO.getSort().toString());
            phase.setControlMode(phasePO.getControlMode().toString());
            phase.setMinGreenTime(phasePO.getMinGreenTime().toString());
            phase.setMaxGreenTime(phasePO.getMaxGreenTime().toString());
            phase.setPhaseTime(phasePO.getPhaseTime().toString());

            Integer oldGreenTime = phasePO.getGreenTime();
            Integer optTime = phaseOffsetTimeMap.get(phaseNo);
            if (optTime != null) {
                Integer newGreenTime = oldGreenTime + optTime;
                phase.setGreenTime(newGreenTime.toString());
            } else {
                phase.setGreenTime(oldGreenTime.toString());
            }

            phase.setGreenFlashTime(phasePO.getGreenFlashTime().toString());
            phase.setPedFlashTime(phasePO.getPedFlashTime().toString());
            phase.setYellowTime(phasePO.getYellowTime().toString());
            phase.setRedTime(phasePO.getRedTime().toString());

            phaseList.add(phase);
        }
        return phaseList;
    }

    @Override
    public void restoreSend(CrossIdBO crossIdBO) throws Exception {
        String crossId = crossIdBO.getCrossId();
        feignProxyService.schemeOptRestore(crossId);
    }

    @Override
    public OptEffectVO optEffect(CrossIdAndDirDTO crossIdAndDirDTO) {
        String crossId = crossIdAndDirDTO.getCrossId();
        OptEffectVO optEffectVO = new OptEffectVO();
        TBaseCrossInfo crossInfoPO = baseCrossInfoMapper.selectByPrimaryKey(crossId);
        optEffectVO.setCrossName(crossInfoPO.getName());
        List<CrossSchemeOptLogPO> maxList = crossSchemeOptLogMapper.selectMaxByCrossId(crossId);
        String optReason = null;
        if (!CollectionUtils.isEmpty(maxList)) {
            CrossSchemeOptLogPO crossSchemeOptLogPO = maxList.get(0);
            optReason = crossSchemeOptLogPO.getOptReason();
            Date endTime = crossSchemeOptLogPO.getEndTime();
            if (ObjectUtil.isEmpty(endTime)) {
                optEffectVO.setStatus(2);
            } else {
                optEffectVO.setStatus(1);
            }
        }
        Integer dir = crossIdAndDirDTO.getDir();
        // 获取当前时间之前45分钟的10位时间戳
        long currentSeconds = DateUtil.currentSeconds();
        long optStartTime = currentSeconds - 2 * 60 * 60;
        //long optStartTime = 1689525900l;
        // 优化开始时间，两个小时内最早时间
        CrossSchemeOptLogPO optStartEntity = crossSchemeOptLogMapper.selectOptStartEntity(crossId, optReason, optStartTime);
        if (Objects.nonNull(optStartEntity)) {
            optEffectVO.setOptStartTime(optStartEntity.getStartTime());
        }
        long preSeconds = currentSeconds - 45 * 60;
        if (ObjectUtil.isEmpty(dir) || dir == 0) {
            // 获取路口历史数据
            List<CrossDataHistPO> crossDataHistPOList =
                    crossDataHistMapper.selectByCrossIdAndTimestamp(crossId, preSeconds);
            if (!CollectionUtils.isEmpty(crossDataHistPOList)) {
                List<OptEffectVO.IndexDelay> indexDelayList = buildIndexDelayList(crossDataHistPOList);
                optEffectVO.setIndexDelayList(indexDelayList);
                List<OptEffectVO.QueueStop> queueStopList = buildQueueStopList(crossDataHistPOList);
                optEffectVO.setQueueStopList(queueStopList);
            }
        } else {
            // 获取路口方向历史数据
            List<CrossDirDataHistPO> crossDirDataHistPOList =
                    crossDirDataHistMapper.selectByCrossIdDirAndTimestamp(crossId, dir, preSeconds);
            if (!CollectionUtils.isEmpty(crossDirDataHistPOList)) {
                List<OptEffectVO.IndexDelay> indexDelayList = buildIndexDelayListDir(crossDirDataHistPOList);
                optEffectVO.setIndexDelayList(indexDelayList);
                List<OptEffectVO.QueueStop> queueStopList = buildQueueStopListDir(crossDirDataHistPOList);
                optEffectVO.setQueueStopList(queueStopList);
            }
        }
        return optEffectVO;
    }

    @Override
    public SaveLaneInfoDTO listLaneInfo(CrossIdBO crossIdBO) {
        String crossId = crossIdBO.getCrossId();
        SaveLaneInfoDTO saveLaneInfoDTO = new SaveLaneInfoDTO();
        saveLaneInfoDTO.setCrossId(crossId);
        // 路口拥堵指数
        CrossDataRealtimePO crossDataRealtimePO = crossDataRealtimeMapper.selectByCrossId(crossId);
        Double trafficIndex = crossDataRealtimePO.getTrafficIndex();
        saveLaneInfoDTO.setCrossIndex(Double.valueOf(String.format("%.2f", trafficIndex)));
        // 构造dirList
        buildDirList(crossId, saveLaneInfoDTO);
        // 构造ledConfigList
        buildLedConfigList(crossId, saveLaneInfoDTO);

        return saveLaneInfoDTO;
    }

    private void buildLedConfigList(String crossId, SaveLaneInfoDTO saveLaneInfoDTO) {
        saveLaneInfoDTO.setLedConfigList(new ArrayList<>());
        List<LedConfigListElement> ledConfigList = saveLaneInfoDTO.getLedConfigList();
        // 获取灯组数据
        List<BaseCrossLightsPO> crossLightsPOList = baseCrossLightsMapper.selectByCrossId(crossId);
        // 赋值给ledConfigList
        for (BaseCrossLightsPO crossLightsPO : crossLightsPOList) {
            LedConfigListElement ledConfigListElement = new LedConfigListElement();
            ledConfigListElement.setId(crossLightsPO.getId());
            ledConfigListElement.setOrder(crossLightsPO.getSort());
            ledConfigListElement.setCode(crossLightsPO.getLightsNo());
            // 处理type
            Integer type = crossLightsPO.getType();
            if (type > 0 && type < 10) { // 机动车灯
                ledConfigListElement.setType(1);
                Integer signalType = MOTOR_VEH_MAP.get(type);
                ledConfigListElement.setSignalType(signalType);
            } else if (type >= 10 && type < 20) { // 非机动车灯
                ledConfigListElement.setType(2);
                Integer signalType = NON_MOTOR_VEH_MAP.get(type);
                ledConfigListElement.setSignalType(signalType);
            } else if (type >= 20 && type < 30) { // 行人灯
                ledConfigListElement.setType(4);
                ledConfigListElement.setSignalType(8);
            } else if (type == 30) { // 公交专用灯
                ledConfigListElement.setType(3);
                ledConfigListElement.setSignalType(7);
            }
            // 灯组放行方向
            Integer dir = crossLightsPO.getDir();
            if (type < 20 || type >= 30) { // 非行人灯
                String address = LightsAddressEnum.getMsgByCode(dir);
                ledConfigListElement.setAddress(address);
            } else if (type == 20) { // 一次过街
                String s = "" + dir + 1;
                setLightAddress(ledConfigListElement, s);
            } else if (type == 22) { // 二次过街-出口
                String s = "" + dir + 21;
                setLightAddress(ledConfigListElement, s);
            } else if (type == 21) { // 二次过街-进口
                String s = "" + dir + 22;
                setLightAddress(ledConfigListElement, s);
            }
            ledConfigList.add(ledConfigListElement);
        }
        Collections.sort(ledConfigList, Comparator.comparingInt(led -> Integer.valueOf(led.getCode())));
    }

    private void setLightAddress(LedConfigListElement ledConfigListElement, String s) {
        int i = Integer.parseInt(s);
        String address = LightsAddressEnum.getMsgByCode(i);
        ledConfigListElement.setAddress(address);
    }

    private void buildDirList(String crossId, SaveLaneInfoDTO saveLaneInfoDTO) {
        saveLaneInfoDTO.setDirList(new ArrayList<>());
        List<LaneInfoPO> laneInfoPOList = laneInfoMapper.selectBycrossId(crossId);
        Map<Integer, List<LaneInfoPO>> collect = laneInfoPOList.stream()
                .collect(Collectors.groupingBy(LaneInfoPO::getDir));
        for (Map.Entry<Integer, List<LaneInfoPO>> entry : collect.entrySet()) {
            Integer key = entry.getKey();
            DirListElement dirListElement = new DirListElement();
            dirListElement.setDir(key);
            // 查询拥堵状态、排队长度和指数
            CrossDirDataRealtimePO crossDirDataRealtimePO =
                    crossDirDataRealtimeMapper.selectByCrossIdAndDirType(crossId, key);
            if (crossDirDataRealtimePO != null) {
                dirListElement.setStatus(crossDirDataRealtimePO.getStatus());
                dirListElement.setTraffic_index(crossDirDataRealtimePO.getTrafficIndex());
                dirListElement.setLength(crossDirDataRealtimePO.getLength());
                dirListElement.setDuration(crossDirDataRealtimePO.getDuration());
            }
            // 获取是否有行人道
            BaseCrossDirInfoPO baseCrossDirInfoPO = baseCrossDirInfoMapper.selectByCrossIdAndDirType(crossId, key);
            Integer isPedestrian = baseCrossDirInfoPO.getIsPedestrian();
            dirListElement.setIsPersonCross(isPedestrian);
            // 构造laneList
            List<LaneInfoPO> value = entry.getValue();
            List<LaneInfoPO> sortedValue = value.stream().
                    sorted(Comparator.comparing(LaneInfoPO::getSort)).
                    collect(Collectors.toList());
            List<LaneListElement> laneListElementList = new ArrayList<>();
            for (LaneInfoPO laneInfoPO : sortedValue) {
                LaneListElement laneListElement = new LaneListElement();
                String laneId = laneInfoPO.getId();
                laneListElement.setId(laneId);
                Integer turn = laneInfoPO.getTurn();
                laneListElement.setDirection(turn);
                laneListElement.setLaneType(laneInfoPO.getCategory());
                laneListElement.setName(laneInfoPO.getCode());
                // 根据laneId获取灯组代码
                String lightsCode = baseCrossLaneLightsMapper.selectLightsCodeByLaneId(laneId);
                if (lightsCode != null) {
                    laneListElement.setLedNum(lightsCode);
                }
                laneListElementList.add(laneListElement);
            }
            dirListElement.setLaneList(laneListElementList);
            // 构造行人过街数据
            List<BaseCrossLightsPO> crossLightsPOList = baseCrossLightsMapper.selectByCrossIdAndDir(crossId, key);
            for (BaseCrossLightsPO crossLightsPO : crossLightsPOList) {
                Integer type = crossLightsPO.getType();
                String lightsNo = crossLightsPO.getLightsNo();
                if (type == 20) { // 一次过街
                    dirListElement.setPersonCrossType("oneCross");
                    dirListElement.setOneCross(lightsNo);
                } else if (type == 22) { // 二次过街出口灯
                    dirListElement.setPersonCrossType("twiceCross");
                    dirListElement.setTwiceCrossOut(lightsNo);
                } else if (type == 21) { // 二次过街进口灯
                    dirListElement.setPersonCrossType("twiceCross");
                    dirListElement.setTwiceCrossIn(lightsNo);
                }
            }
            List<DirListElement> dirList = saveLaneInfoDTO.getDirList();
            dirList.add(dirListElement);
        }
        // 返回没有车道的方向
        List<BaseCrossDirInfoPO> baseCrossDirInfoPOList = baseCrossDirInfoMapper.selectByCrossId(crossId);
        for (BaseCrossDirInfoPO baseCrossDirInfoPO : baseCrossDirInfoPOList) {
            List<DirListElement> dirList = saveLaneInfoDTO.getDirList();
            Integer dirType = baseCrossDirInfoPO.getDirType();
            List<LaneInfoPO> laneInfoPOList1 = laneInfoMapper.selectInDirByCrossIdAndDir(crossId, dirType);
            if (CollectionUtils.isEmpty(laneInfoPOList1)) {
                DirListElement dirListElement = new DirListElement();
                dirListElement.setDir(baseCrossDirInfoPO.getDirType());
                dirListElement.setIsPersonCross(baseCrossDirInfoPO.getIsPedestrian());
                dirList.add(dirListElement);
            }
        }
    }

    private List<OptEffectVO.QueueStop> buildQueueStopListDir(List<CrossDirDataHistPO> crossDirDataHistPOList) {
        List<OptEffectVO.QueueStop> queueStopList = new ArrayList<>();
        for (CrossDirDataHistPO crossDirDataHistPO : crossDirDataHistPOList) {
            OptEffectVO.QueueStop queueStop = new OptEffectVO.QueueStop();
            // 获取采集时间
            long batchTimeLong = crossDirDataHistPO.getBatchTime();
            batchTimeLong *= 1000;
            Date date = DateUtil.date(batchTimeLong);
            queueStop.setTime(date);
            queueStop.setQueue(crossDirDataHistPO.getQueueLength());
            queueStop.setStop(crossDirDataHistPO.getStopTimes());
            queueStopList.add(queueStop);
        }
        return queueStopList;
    }

    private List<OptEffectVO.IndexDelay> buildIndexDelayListDir(List<CrossDirDataHistPO> crossDirDataHistPOList) {
        List<OptEffectVO.IndexDelay> indexDelayList = new ArrayList<>();
        for (CrossDirDataHistPO crossDirDataHistPO : crossDirDataHistPOList) {
            OptEffectVO.IndexDelay indexDelay = new OptEffectVO.IndexDelay();
            // 获取采集时间
            long batchTimeLong = crossDirDataHistPO.getBatchTime();
            batchTimeLong *= 1000;
            Date date = DateUtil.date(batchTimeLong);
            indexDelay.setTime(date);
            indexDelay.setIndex(crossDirDataHistPO.getTrafficIndex());
            indexDelay.setDelay(crossDirDataHistPO.getDelayTime());
            indexDelayList.add(indexDelay);
        }
        return indexDelayList;
    }

    private List<OptEffectVO.QueueStop> buildQueueStopList(List<CrossDataHistPO> crossDataHistPOList) {
        List<OptEffectVO.QueueStop> queueStopList = new ArrayList<>();
        for (CrossDataHistPO crossDataHistPO : crossDataHistPOList) {
            OptEffectVO.QueueStop queueStop = new OptEffectVO.QueueStop();
            // 获取采集时间
            Integer batchTime = crossDataHistPO.getBatchTime();
            Date date = DateUtil.date(batchTime * 1000);
            queueStop.setTime(date);
            queueStop.setQueue(crossDataHistPO.getQueueLength());
            queueStop.setStop(crossDataHistPO.getStopTimes());
            queueStopList.add(queueStop);
        }
        return queueStopList;
    }

    private List<OptEffectVO.IndexDelay> buildIndexDelayList(List<CrossDataHistPO> crossDataHistPOList) {
        List<OptEffectVO.IndexDelay> indexDelayList = new ArrayList<>();
        for (CrossDataHistPO crossDataHistPO : crossDataHistPOList) {
            OptEffectVO.IndexDelay indexDelay = new OptEffectVO.IndexDelay();
            // 获取采集时间
            Integer batchTime = crossDataHistPO.getBatchTime();
            Date date = DateUtil.date(batchTime * 1000);
            indexDelay.setTime(date);
            indexDelay.setIndex(crossDataHistPO.getTrafficIndex());
            indexDelay.setDelay(crossDataHistPO.getDelayTime());
            indexDelayList.add(indexDelay);
        }
        return indexDelayList;
    }

    private Integer getNewGreenTime(String crossId, Integer schemeId, String phaseNo) {
        CrossPhasePO entity = new CrossPhasePO();
        entity.setCrossId(crossId);
        entity.setPlanId(schemeId);
        List<CrossPhasePO> crossPhasePOList = baseCrossPhaseMapper.listCrossPhasePO(entity);
        for (CrossPhasePO crossPhasePO : crossPhasePOList) {
            String newPhaseNo = crossPhasePO.getPhaseNo();
            if (Objects.equals(phaseNo, newPhaseNo)) {
                return crossPhasePO.getGreenTime();
            }
        }
        return 0;
    }

    private List<SchemeOptVO.PhaseOptTime> buildPhaseOptTimeList(
            Collection<CrossSchemeOptLogPO> crossSchemeOptLogPOList, Integer schemeId, String crossId) {
        // 初始化列表
        List<SchemeOptVO.PhaseOptTime> phaseOptTimeList = new ArrayList<>();
        initPhaseOptTimeList(schemeId, phaseOptTimeList, crossId);
        for (CrossSchemeOptLogPO crossSchemeOptLogPO : crossSchemeOptLogPOList) {
            for (SchemeOptVO.PhaseOptTime phaseOptTime : phaseOptTimeList) {
                String phaseNo1 = crossSchemeOptLogPO.getPhaseNo();
                String phaseNo2 = phaseOptTime.getPhaseNo();
                if (Objects.equals(phaseNo1, phaseNo2)) {
                    int optTime = crossSchemeOptLogPO.getOptTime();
                    int oriGreenTime = crossSchemeOptLogPO.getOriGreenTime();
                    phaseOptTime.setOriGreenTime(oriGreenTime);
                    phaseOptTime.setOptGreenTime(oriGreenTime + optTime);
                    phaseOptTime.setOptTime(optTime);
                }
            }
        }
        return phaseOptTimeList;
    }

    private void initPhaseOptTimeList(Integer schemeId, List<SchemeOptVO.PhaseOptTime> phaseOptTimeList, String crossId) {
        CrossPhasePO phasePO = new CrossPhasePO();
        phasePO.setCrossId(crossId);
        phasePO.setPlanId(schemeId);
        List<CrossPhasePO> crossPhasePOList = baseCrossPhaseMapper.listCrossPhasePO(phasePO);
        for (CrossPhasePO crossPhasePO : crossPhasePOList) {
            SchemeOptVO.PhaseOptTime phaseOptTime = new SchemeOptVO.PhaseOptTime();
            phaseOptTime.setPhaseNo(crossPhasePO.getPhaseNo());
            phaseOptTime.setOriGreenTime(crossPhasePO.getGreenTime());
            phaseOptTime.setOptGreenTime(crossPhasePO.getGreenTime());
            phaseOptTime.setOptTime(0);
            phaseOptTimeList.add(phaseOptTime);
        }
    }
}
