package net.wanji.opt.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.enums.BaseEnum;
import net.wanji.common.enums.CrossInOutEnum;
import net.wanji.common.enums.StrategyAndMetricsEnum;
import net.wanji.common.enums.TurnConvertEnum;
import net.wanji.common.framework.exception.DubboProviderException;
import net.wanji.common.utils.tool.CrossUtil;
import net.wanji.common.utils.tool.DateUtil;
import net.wanji.common.utils.tool.TimeArrayUtil;
import net.wanji.databus.bo.CrossIdBO;
import net.wanji.databus.dao.entity.BaseCrossSchedulesPO;
import net.wanji.databus.dao.entity.BaseCrossSchedulesPlanPO;
import net.wanji.databus.dao.entity.BaseCrossSchemePO;
import net.wanji.databus.dao.entity.CrossSectionPO;
import net.wanji.databus.dao.mapper.BaseCrossDirInfoMapper;
import net.wanji.databus.dao.mapper.BaseCrossSchedulesMapper;
import net.wanji.databus.dao.mapper.BaseCrossSchedulesPlanMapper;
import net.wanji.databus.dao.mapper.BaseCrossSchemeMapper;
import net.wanji.databus.dao.mapper.BaseCrossSectionMapper;
import net.wanji.databus.dao.mapper.BaseCrossTurnInfoMapper;
import net.wanji.databus.dao.mapper.CrossDataHistMapper;
import net.wanji.databus.dao.mapper.CrossDataRealtimeMapper;
import net.wanji.databus.dao.mapper.CrossDirDataHistMapper;
import net.wanji.databus.dao.mapper.CrossTurnDataHistMapper;
import net.wanji.databus.po.CrossDataHistPO;
import net.wanji.databus.po.CrossDirDataHistPO;
import net.wanji.databus.po.CrossDirDataHistPOExt;
import net.wanji.databus.po.CrossTurnDataHistPO;
import net.wanji.databus.po.CrossTurnDataHistPOExt;
import net.wanji.opt.bo.CrossSchemeListBO;
import net.wanji.opt.bo.CurveChartBO;
import net.wanji.opt.bo.CurveChartBO.DirTurn;
import net.wanji.opt.bo.ProblemSchemeBO;
import net.wanji.opt.bo.SchemeDetailOverallBO;
import net.wanji.opt.common.EsDateIndexUtil;
import net.wanji.opt.common.enums.TimeGranularityEnum;
import net.wanji.opt.dao.mapper.HoloEventMapper;
import net.wanji.opt.dao.mapper.base.BaseRidInfoMapper;
import net.wanji.opt.dao.mapper.strategy.SceneMapper;
import net.wanji.opt.dao.mapper.strategy.SceneStrategyMapper;
import net.wanji.opt.dao.mapper.strategy.StrategyMapper;
import net.wanji.opt.entity.EventAlarmInfo;
import net.wanji.opt.po.base.BaseRidInfo;
import net.wanji.opt.po.base.CrossLaneDataHistPoExtend;
import net.wanji.opt.po.base.EventStatisticPo;
import net.wanji.opt.po.strategy.ScenePO;
import net.wanji.opt.po.strategy.SceneStrategyPO;
import net.wanji.opt.po.strategy.StrategyPO;
import net.wanji.opt.po.trend.EventInfoSimplePo;
import net.wanji.opt.po.trend.HoloEventInfoPO;
import net.wanji.opt.service.EventService;
import net.wanji.opt.service.SchemeEvaluateService;
import net.wanji.opt.vo.RunningEvaluateMetricsDetailVO;
import net.wanji.opt.vo.SchemeEvaluateCrossSchemeListVO;
import net.wanji.opt.vo.SchemeEvaluateCurveChartVO;
import net.wanji.opt.vo.SchemeEvaluateProblemSchemeVO;
import net.wanji.opt.vo.SchemeEvaluateSchemeDetailOverallVO;
import net.wanji.opt.vo.SchemeEvaluateSchemeDetailedProblemVO;
import net.wanji.opt.vo.SchemeEvaluateStrategyMetricMenuVO;
import org.jetbrains.annotations.NotNull;
import org.joda.time.DateTime;
import org.joda.time.Minutes;
import org.joda.time.Seconds;
import org.joda.time.format.DateTimeFormat;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.lang.reflect.Field;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Comparator;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Optional;
import java.util.OptionalDouble;
import java.util.OptionalInt;
import java.util.Set;
import java.util.TreeMap;
import java.util.TreeSet;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 *
 */
@Slf4j
@Service
public class EventServiceImpl implements EventService {
    @Resource
    HoloEventMapper holoEventMapper;

    @Resource
    BaseRidInfoMapper baseRidInfoMapper;


    @Override
    public EventStatisticPo findCrossEventCount(String crossId, String type, String startTime, String endTime) throws DubboProviderException {
        Map<String, Object> params = new HashMap<>();
        params.put("crossId", crossId);
        params.put("startDate", startTime);
        params.put("endDate", endTime);
        params.put("eventType", type);

        EventStatisticPo po = holoEventMapper.findCrossEventCount(params);

        if (po != null) {
            String dirs = po.getDirs();
            Map<String, Long> countMap = null;
            List<Map<String, Object>> list = new ArrayList<>();
            if (Objects.nonNull(dirs)) {
                String[] sps = dirs.split(",");
                //统计每个方向发生的次数
                countMap = Arrays.stream(sps).collect(Collectors.groupingBy(String::valueOf, Collectors.counting()));
                countMap.entrySet().forEach(o -> {
                    Map<String, Object> map = new HashMap<>();
                    map.put("dir", o.getKey());
                    map.put("count", o.getValue());
                    list.add(map);
                });
            }
            po.setDirEventCount(countMap);
            po.setDirEventCountList(list);


            QueryWrapper<BaseRidInfo> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("end_cross_id", crossId);

            List<BaseRidInfo> ridList = baseRidInfoMapper.selectList(queryWrapper);

            Map<String, List<String>> group = ridList.stream().collect(Collectors.groupingBy(o -> o.getInDir().toString(), Collectors.mapping(BaseRidInfo::getWkt, Collectors.toList())));

            //进口道方向wkt
            Map<String, String> dirWkt = new HashMap<>();
            group.forEach((k, v) -> {
                dirWkt.put(k, v.get(0));
            });
            po.setDirs(null);
            po.setDirWkt(dirWkt);
        }
        return po;
    }

    @Override
    public List<Map<String, Object>> findEventDistribute(String crossId, String startTime, String endTime) {
        Map<String, Object> params = new HashMap<>();
        params.put("crossId", crossId);
        params.put("startDate", startTime);
        params.put("endDate", endTime);

        List<Map<String, Object>> retList = new ArrayList<>();

        DateTime start = DateTime.parse(startTime, DateTimeFormat.forPattern(EsDateIndexUtil.YMD_HM_FORMATTER));
        DateTime end = DateTime.parse(endTime, DateTimeFormat.forPattern(EsDateIndexUtil.YMD_HM_FORMATTER));
        //存放时段
        Set<String> sortedSet = getTimeScopeList(start, end, EsDateIndexUtil.YMD_HM_FORMATTER);

        List<EventInfoSimplePo> list = holoEventMapper.findEventDistribute(params);
        //每5分组分组
        Map<String, List<EventInfoSimplePo>> group = list.stream().collect(Collectors.groupingBy(o -> o.getTimeAxisStart()));

        List<EventInfoSimplePo> tmpList = new ArrayList<>();
        List<String> dataserList = new ArrayList<>();

        group.forEach((time, valList1) -> {
            //5分组内数据按类型分组
            Map<String, List<EventInfoSimplePo>> groupByType = valList1.stream().collect(Collectors.groupingBy(o -> o.getEventType()));

            //List<EventInfoSimplePo> tmpList = new ArrayList<>();

            groupByType.forEach((type, valList2) -> {
                EventInfoSimplePo tmp = null;
                for (EventInfoSimplePo po : valList2) {
                    po.setDate(net.wanji.common.utils.tool.DateUtil.formatDate(po.getStartTime(), "M/d"));
                    //合并时间重叠事件
                    if (tmp != null) {
                        Date agoStartTime = tmp.getStartTime();
                        Date agoEndTime = tmp.getEndTime();
                        Date currentStartTime = po.getStartTime();
                        Date currentEndTime = po.getEndTime();
                        if (currentStartTime.before(agoEndTime)) {
                            int duration = Seconds.secondsBetween(new DateTime(agoStartTime.getTime()), new DateTime(currentEndTime.getTime())).getSeconds();
                            po.setDuration(duration);
                            po.setStartTime(agoStartTime);
                            po.setEndTime(currentEndTime);
                            po.setTimeAxisStart(tmp.getTimeAxisStart());
                            po.setCount(po.getCount() + 1);
                            //清理合并前的记录
                            tmpList.remove(tmp);
                        }
                    }
                    //==============周期数据时间范围============================================//
                    DateTime startAxis = new DateTime(DateTime.parse(po.getTimeAxisStart(), DateTimeFormat.forPattern("yyyy-MM-dd HH:mm:ss")));
                    DateTime endAxis = new DateTime(po.getEndTime());
                    int diffNum = Minutes.minutesBetween(startAxis, endAxis).getMinutes();

                    List<String> dataTimeList = po.getDataTimeList();
                    for (int i = 0; i <= diffNum; i++) {
                        DateTime axisDateTime = startAxis.plusMinutes(i * 1);
                        int m = axisDateTime.getMinuteOfHour();
                        if (m % 5 == 0 && axisDateTime.isBefore(endAxis)) {
                            dataTimeList.add(axisDateTime.toString("yyyy-MM-dd HH:mm:ss"));
                        }
                    }
                    po.setTimeAxisEnd(endAxis.toString("yyyy-MM-dd HH:mm:ss"));
                    po.setDataTimeList(dataTimeList);
                    //======================================================================//
                    tmpList.add(po);
                    tmp = po;
                }
            });
        });
        List<EventInfoSimplePo> sortList = tmpList.stream().sorted(Comparator.comparing(o -> o.getStartTime())).collect(Collectors.toList());
        sortList.forEach(po -> {
            String axis = DateUtil.formatDate(po.getStartTime(), "M/d") + "," + DateUtil.formatDate(po.getStartTime(), "HH:mm:ss") + "," + po.getEventType();
            dataserList.add(axis);
        });
        Map<String, Object> map = new HashMap<>();
        map.put("statusByDateList", sortList);
        map.put("dataser", dataserList);
        retList.add(map);
//        Set<String> timeList = group.keySet();
        //补充缺少时段数据，保留时段字段默认值
//        for (String timeSec : sortedSet) {
//            if (!timeList.contains(timeSec)) {
//                Map<String, Object> map = new HashMap<>();
//                map.put("timeAxis", timeSec);
//                map.put("statusByDateList", new ArrayList<>());
//                retList.add(map);
//            }
//        }

//        List<Map<String, Object>> sortList = retList.stream().sorted(Comparator.comparing(o -> o.get("timeAxis").toString())).collect(Collectors.toList());

        return retList;
    }

    @Override
    public List<EventAlarmInfo> findEventAlarmInfo(String realtimeType, String eventType, String name) throws DubboProviderException {
        //获取数据库数据
        List<EventAlarmInfo> eventAlarmInfo = holoEventMapper.findEventAlarmInfo(realtimeType, eventType, name);

        return eventAlarmInfo;
    }


    public Set<String> getTimeScopeList(DateTime start, DateTime end, String dateFormat) {
        Set<String> indexList = new HashSet<>();
        int diffNum = Minutes.minutesBetween(start, end).getMinutes();
        int unitTime = 5;
        for (int i = 0; i <= diffNum; i = i + unitTime) {
            String format = start.plusMinutes(i).toString(dateFormat);
            indexList.add(format);
        }
        return indexList;
    }
}
